/**********************************************************************
 *
 * Copyright (c) 2004 Olaf Willuhn
 * All rights reserved.
 * 
 * This software is copyrighted work licensed under the terms of the
 * Jameica License.  Please consult the file "LICENSE" for details. 
 *
 **********************************************************************/
package de.willuhn.jameica.hbci.passports.ddv.rmi;


/**
 * Passport fuer das Sicherheitsmedium "Chipkarte" (DDV).
 */
public interface Passport extends de.willuhn.jameica.hbci.passport.Passport {

}
