/**********************************************************************
 *
 * Copyright (c) 2024 Olaf Willuhn
 * All rights reserved.
 * 
 * This software is copyrighted work licensed under the terms of the
 * Jameica License.  Please consult the file "LICENSE" for details. 
 *
 **********************************************************************/

package de.willuhn.jameica.hbci.messaging;

import de.willuhn.jameica.messaging.Message;

/**
 * Wird gesnedet, wenn die Saldo-Limits neu berechnet werden sollen.
 */
public class SaldoLimitsMessage implements Message
{

}


